--[[
    Aaron MacDonald
    November 25, 2010
]]

require('gfx')
require('colour_cycle')
require('particles')
require('utility')

require('starfield')

require('ship')
require('bullet')

require('data.particles')
require('data.ships')

----------------------------------------------------------------------

local FPS = 60
local SECONDS_PER_FRAME = 1 / FPS

local accumulatedTime

local isPaused

local KEY_QUIT = 'escape'
local KEY_PAUSE = 'p'

----------------------------------------------------------------------

local KEY_UP = 'w'
local KEY_LEFT = 'a'
local KEY_DOWN = 's'
local KEY_RIGHT = 'd'

----------------------------------------------------------------------

local MOUSE_SPRITE_NAME = 'target'
local MOUSE_SPRITE
local MOUSE_COLOUR_CYCLE

local MOUSE_COLOURS = { {255, 255, 255},
                        {255, 225,  64} }
local MOUSE_COLOUR_SPEED = 0.1

----------------------------------------------------------------------

local ARENA_WIDTH
local ARENA_HEIGHT

----------------------------------------------------------------------

local SPARKLE_SPRITE_NAME = 'sparkle'
local NUM_GUNSIGHT_SPARKLES = 50
local SPARKLE_FADE_STEP = 0.05

local SPARKLE_START_COLOUR = {128, 191, 255}
local SPARKLE_END_COLOUR = {255, 255, 255}

local sparkleSprite
local sparkles

----------------------------------------------------------------------

local playerType = 'sphinx'
local enemyType = 'terminus'

local playerShip
local enemyShip

local allBullets

----------------------------------------------------------------------

function initAimSparkles()
    sparkles = {}
    for i = 1, NUM_GUNSIGHT_SPARKLES do
        sparkles[i] = {
            range = math.random() * playerShip.type.bulletRange,
            fade = math.random() * 2
        }
    end
end

function love.load()
    math.randomseed( os.time() )

    accumulatedTime = 0
    isPaused = false

    ----------------------------

    starfield.init()

    ----------------------------

    love.mouse.setVisible(false)

    MOUSE_SPRITE = gfx.createSprite(MOUSE_SPRITE_NAME)

    MOUSE_COLOUR_CYCLE = colourCycle.createCycle(MOUSE_COLOURS,
      MOUSE_COLOUR_SPEED)

    ----------------------------

    allBullets = {}

    ARENA_WIDTH = love.graphics.getWidth()
    ARENA_HEIGHT = love.graphics.getHeight()

    local y = ARENA_HEIGHT * 0.5

    local x = ARENA_WIDTH * 0.25

    playerShip = ship.create(SHIPS[playerType], x, y, 0)

    sparkleSprite = gfx.createSprite(SPARKLE_SPRITE_NAME)
    initAimSparkles()

    local x = ARENA_WIDTH * 0.75

    enemyShip = ship.create(SHIPS[enemyType], x, y, math.pi)
end

----------------------------------------------------------------------

function love.keypressed( key, unicode )
    if key == KEY_QUIT then
        love.event.push('q')
    elseif key == KEY_PAUSE then
        isPaused = not isPaused
    end
end

----------------------------------------------------------------------

function love.focus(hasFocus)
    if not hasFocus then
        isPaused = true
    end
end

----------------------------------------------------------------------

local function updateAimSparkles()
    for _, s in ipairs(sparkles) do
        s.fade = s.fade - SPARKLE_FADE_STEP
        if s.fade < 0 then
            s.range = math.random() * playerShip.type.bulletRange
            s.fade = s.fade + 2
        end
    end
end

local function updateBullets()
    local toRemove = {}
    local numRemoved = 0

    local cx, cy, x, y

    for i, b in ipairs(allBullets) do
        local alive = bullets.update(b, SECONDS_PER_FRAME,
          ARENA_WIDTH, ARENA_HEIGHT)

        if not alive then
            table.insert(toRemove, i)
        end
    end

    for _, bi in ipairs(toRemove) do
        table.remove(bullets, bi - numRemoved)
        numRemoved = numRemoved + 1
    end
end

local function doUpdate()
    starfield.update()
    colourCycle.update(MOUSE_COLOUR_CYCLE)

    ship.updateVelocity(playerShip,
      love.keyboard.isDown(KEY_UP),
      love.keyboard.isDown(KEY_DOWN),
      love.keyboard.isDown(KEY_LEFT),
      love.keyboard.isDown(KEY_RIGHT)
    )

    ship.turnTowardsTarget(enemyShip, playerShip.x, playerShip.y)
    ship.turnTowardsTarget(playerShip,
      love.mouse.getX(), love.mouse.getY())

    ship.update(enemyShip, ARENA_WIDTH, ARENA_HEIGHT,
      SECONDS_PER_FRAME)
    ship.update(playerShip, ARENA_WIDTH, ARENA_HEIGHT,
      SECONDS_PER_FRAME)

    if love.mouse.isDown('l') then
        ship.shoot(playerShip, allBullets)
    end

    updateAimSparkles()

    updateBullets()
end

function love.update(dt)
    accumulatedTime = accumulatedTime + dt
    while accumulatedTime >= SECONDS_PER_FRAME do
        if not isPaused then
            doUpdate()
        end
        accumulatedTime = accumulatedTime - SECONDS_PER_FRAME
    end
end

----------------------------------------------------------------------

local function drawAimSparkles()
    for _, s in ipairs(sparkles) do
        local x, y = utility.polarToCartesian(playerShip.angle, s.range,
          playerShip.x, playerShip.y)

        local fade
        if s.fade < 1 then
            fade = s.fade
        else
            fade = 1 - s.fade
        end

        local sparkleColour = gfx.interpolatedColour(
          SPARKLE_END_COLOUR, SPARKLE_START_COLOUR, fade)

        love.graphics.setColor(sparkleColour[1], sparkleColour[2],
          sparkleColour[3], 255 * fade)

        gfx.drawWrappedSprite(sparkleSprite, x, y,
          ARENA_WIDTH, ARENA_HEIGHT)
    end

    gfx.setColourWhite()
end

local function drawBullets()
    for _, b in ipairs(allBullets) do
        bullets.draw(b, ARENA_WIDTH, ARENA_HEIGHT)
    end
end

local function drawMouse()
    colourCycle.setColour(MOUSE_COLOUR_CYCLE)

    gfx.drawSprite(MOUSE_SPRITE,
      love.mouse.getX(), love.mouse.getY())

    gfx.setColourWhite()
end

function love.draw()
    starfield.draw()

    drawAimSparkles()

    ship.draw(enemyShip, ARENA_WIDTH, ARENA_HEIGHT)
    ship.draw(playerShip, ARENA_WIDTH, ARENA_HEIGHT)

    drawBullets()

    drawMouse()
end